
# q-expansion of newform 1280.4.d.n, downloaded from the LMFDB on 02 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1280
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[511, [1, 0]], [261, [-1, 0]], [257, [1, 0]]]
aps_data = [[0, 0], [0, 4], [0, -5], [16, 0], [0, 60], [0, 86], [18, 0], [0, 44], [-48, 0], [0, -186], [176, 0], [0, -254], [-186, 0], [0, 100], [168, 0], [0, 498], [0, 252], [0, -58], [0, -1036], [-168, 0], [-506, 0], [272, 0], [0, 948], [1014, 0], [-766, 0], [0, 1314], [448, 0], [0, -1548], [0, 278], [-558, 0], [344, 0], [0, 780], [-666, 0], [0, -884], [0, 114], [40, 0], [0, -154], [0, 2180], [-3696, 0], [0, 1302], [0, -4308], [0, -1550], [48, 0], [1058, 0], [0, 3714], [1768, 0], [0, -4036], [680, 0], [0, 2388], [0, 3874], [-3162, 0], [5424, 0], [-3886, 0], [0, 5100], [2178, 0], [6144, 0], [0, 822], [8480, 0], [0, 1138], [-5706, 0], [0, 3028], [0, -3390], [0, -4156], [-6552, 0], [1366, 0], [0, 2598], [0, 3292], [6194, 0], [0, 10020], [0, -3130], [4194, 0], [4104, 0], [7496, 0], [0, 5842], [0, 412], [2568, 0], [0, -13086], [0, 10454], [-10830, 0], [8566, 0], [0, 13884], [0, -4286], [6336, 0], [-8974, 0], [2968, 0], [0, 12372], [11394, 0], [358, 0], [0, -7530], [-13768, 0], [0, 13380], [-6336, 0], [5008, 0], [0, -12900], [0, -8116], [4944, 0], [0, -5466], [-10074, 0], [0, 13828], [0, -15226], [0, -13228], [0, -8490], [0, -10284], [-1770, 0], [0, -6068], [21506, 0], [0, -12108], [15474, 0], [2520, 0], [12790, 0], [11576, 0], [0, -20126], [27942, 0], [0, 22540], [5128, 0], [-12798, 0], [0, -21148], [-16464, 0], [0, -24234], [0, -22836], [0, -26318], [28802, 0], [0, -2526], [0, 23076], [0, 7868], [0, 21510], [0, -30014], [-816, 0], [9952, 0], [0, -33946], [0, 23420], [14592, 0], [9056, 0], [0, 17554], [36438, 0], [-9022, 0], [0, -1470], [0, 5252], [0, 12294], [-15546, 0], [0, -19364], [0, 7314], [-11984, 0], [0, -13500], [0, -44602], [-35448, 0], [0, -12590], [-24906, 0], [0, -23204], [19848, 0], [0, 27542], [-20718, 0], [0, 25172], [12864, 0], [0, 23092], [-14208, 0], [26584, 0], [162, 0], [29734, 0], [0, 17142], [0, 26436], [-27882, 0], [-12656, 0], [0, -2916], [-6894, 0], [45264, 0], [52016, 0], [0, 13858], [4370, 0], [0, 36786], [0, 57756], [0, 6374], [22008, 0], [-40826, 0], [-37264, 0], [-26058, 0], [0, -12308], [0, 27426], [4000, 0], [0, -41194], [69176, 0], [0, -24372], [0, 34306], [42630, 0], [50904, 0], [0, -67950], [0, 47078], [0, 32132], [11782, 0], [-20208, 0], [71234, 0], [0, -22812], [0, -70180], [0, 12390], [0, -59532], [-39642, 0], [47954, 0], [0, 51878], [-3390, 0], [-25440, 0], [0, 55254], [-59488, 0], [0, -6446], [3746, 0], [0, 46380], [0, 40518], [9728, 0], [0, 43620], [-60186, 0], [0, -76196], [23762, 0], [0, -82062], [60688, 0], [0, 51972], [-67608, 0], [51910, 0], [14120, 0], [2418, 0], [93504, 0], [0, -68346], [0, -77246], [78712, 0], [20802, 0], [-72232, 0], [0, 27972], [0, 27058], [-16170, 0], [64800, 0], [-19856, 0], [0, 79164], [0, -66058], [0, 56972], [-2512, 0], [74406, 0], [0, -85916], [-94200, 0], [14258, 0], [0, 77394], [0, -46020], [20760, 0], [0, 65172], [0, 27244], [55840, 0], [0, 15830], [86130, 0], [57144, 0], [-111880, 0], [0, -33972], [0, -105044], [-9576, 0], [0, -18970], [58818, 0], [-35088, 0], [-23066, 0], [0, 27702], [0, -37332], [0, -121262], [0, 105892], [0, -894], [3670, 0], [30824, 0], [0, 104244], [0, -96446], [0, -42010], [-73086, 0], [0, -78244], [0, -55626], [-89322, 0], [0, 14164], [0, 74274], [0, -102058], [0, -4156], [62422, 0], [15776, 0], [123026, 0], [31504, 0], [0, -68316], [0, -51610], [23782, 0], [0, 60732], [99144, 0], [-91496, 0], [14976, 0], [0, 109378], [0, 30772], [83904, 0], [11858, 0], [0, -131310], [81880, 0], [104802, 0], [0, -79722], [0, 75684], [-29514, 0], [0, 63612], [0, 104438], [0, 68262], [-60112, 0], [0, 1554], [0, 82140], [0, 33428], [-139610, 0], [0, 164982], [33488, 0]]
